#!/usr/bin/plsperl
#############################################################################
#
# Personal Library Software 2/15/95
#
# This work is protected by US Copyright Law and contains proprietary and
# confidential trade secrets.  PLS (c) Copyright 1995 by PLS,
# All Rights Reserved
#
# ### pladdsur.pl ###
#
# Function:	Creates remote document and surrogate file for database
#				group in PL/Web.
#
# Created:	3/28/95		Bryan Slavin
#
# Changed:	
#
# Notes:	Adapted from the original pladdsur.pl written by Chris Ekman
##############################################################################


##############################################################################
#
# Name:			mainLine
# Description:	Creates remote documents 
#	The general flow of the program is:
#		1.	Get arguments and process them
#		2.	If -f option is used, get information and create a
#			file with the list of records to be added for pladd
#		3.	Prepare surrogate files (command line and files
#			specified in -f [filename]) 
#		4.	Create a file which contains a list of files to
#			used with pladd
#		5.	Call pladd
#		6.	Substitute the record with a PLS URL in all the
#			surrogate files 
#
# Created:	3/28/95		Bryan Slavin
# Changed:	4/4/95		Bryan Slavin
#				Moved call to adpath so that path can be used in 
#				require statement
#			4/5/95		Bryan Slavin
#				Moved the removal of the temporary file so file is
#				only removed if -f option is used
#			4/10/95		Bryan Slavin
#				Added code that checks if the surrogate file exists.
#				If it does it gives an error message and exits.
#			4/10/95		Bryan Slavin
#				Added -p option which tells pladdsur that the remote
#				file is a pdf file
#			4/21/95		Herman Vandermolen
#				Add interpretation of ~.
#			4/28/95		Bryan Slavin
#				Delete raw file from pdf before assigning variable to 
#				different value. Fixes CR 7632.
#			6/6/95		Bryan Slavin
#				Added check to see if txtext exists.
#			12/4/95		Tien Dinh
#				Took out -g dbgroup option for FastWeb product						
# Returns:	(none)
#
##############################################################################

local		(
		$dbName,		# The name of the database	
		$remList,		# The file with the list of remote docs to add
		$tmpList,		# The file with the list of docs used by pladd
		$minArgs,		# The minimum number of arguments required
		$url,			# The command-line url
		$title,			# The command-line headline
		$surName,		# The command-line surrogate file
		$remName,		# The command-line remote file
		$index,			# Number of files given in $remList
		$i,				# Index for "For" loop
		$pladd,			# Line used to call pladd
		$txtext,		# Line used to call txtext
		$adpath,		# Path to admin directory
		$plsMark,		# PLS markup used for remote docs
		$files,			# Are there any files to add?
		$link)			# URL used in remote docs
			= ("", "", "", "", "", "", "", "", "", "", "", "", "", "", "");
#
# Get path from which pladdsur was executed
#
&getAdPath;

#
#	Get flagged options and arguments
#
require "$adpath" . "getopts.pl";
eval '&Getopts (\'g:f:vip\')';

if ($opt_f){
	$remList = $opt_f;
}


#
#	Set the minimum number of standard arguments
#
if ($opt_i){
	$minArgs = 2;
}
else{
	$minArgs = 3; 
}


#
#	Get the command-line arguments
#
if ($#ARGV < 0){
	print "Not enough arguments\n";
							### EXIT ###
	&surUsage;
}
else {
	$dbName = shift(@ARGV);
}

if ($#ARGV == $minArgs) {
	($url, $title, $surName, $remName) = @ARGV;
	$clArgs = 1;
}
elsif (!($opt_f) && ($#ARGV < $minArgs)) {
	print "Not enough arguments\n";
							### EXIT ###
    &surUsage;
}

$tmpList = $remList . ".new";

$txtext = $adpath . "txtext";

if ($opt_p) {
	die "pladdsur: Can't find txtext" unless ((-e $txtext) || (-e "$txtext.exe"));
}
		

#
#	Get names of all the remote files and prepare them for call to pladd.
#

#
#	Open filename list for pladdsur. Open future filename list for pladd.  
#
$index = 0;
if ($opt_f){

	open (ORIGLIST, "$remList") || die "pladdsur: Can't open $remList";
	open (NEWLIST, ">$tmpList") || die "pladdsur: Can't open $tmpList";

	FILELOOP: while (<ORIGLIST>){

		(@surNames[$index], @remNames[$index], @urls[$index], @hl[$index])  = split(/\s/, $_, 4);
		chop (@hl[$index]);

		@surNames[$index] =~ s/^\~/$ENV{'HOME'}/;
		@remNames[$index] =~ s/^\~/$ENV{'HOME'}/;

		if (-e @surNames[$index]){
			print "Warning:  @surNames[$index] already exists.  Skipping " .
					"this entry in the file.\n";
			next FILELOOP;
		}

		#
		# Check to see if -p was used. If so, execute txtext and set 
		# remname to the ascii file created
		#
		if ($opt_p){
			print "Executing txtext on @remNames[$index]\n" if (opt_v);
			system ("$txtext -f @remNames[$index] -p -1 -o @remNames[$index].raw");
			&filterFile(@remNames[$index]);
			if (!unlink ("@remNames[$index].raw")){
				print "pladdsur: Could not remove temporary file " .
					"@remNames[$index].raw\n";
			}
			@remNames[$index] = "@remNames[$index].txt";
		}

		open (SURFILE, ">@surNames[$index]") || 
				die "pladdsur: Can't open @surNames[$index]";
		open (REMFILE, "@remNames[$index]") || 
				die "pladdsur: Can't open @remNames[$index]";

		print (NEWLIST "@surNames[$index]\n");

		while (<REMFILE>){
			print SURFILE;
		}

		close (REMFILE);
		if ($opt_p) {
			if (!unlink (@remNames[$index])){
				print "pladdsur: Could not remove temporary file " .
						"@remNames[$index]\n";
			}
		}
		close (SURFILE);
		
		if (-z @surNames[$index]){
			unlink (@surNames[$index]);
										### EXIT ###
			die "pladdsur: file is empty.\n";
		}

		$index++;
		$files = 1;
	}

	close (NEWLIST);
	close (ORIGLIST);

}

#
#	If command line arguments were given, process that file as well
#
if ($clArgs){

	#
	#	Check to see if the surrogate file already exists
	#
	if (-e $surName){
		print "Warning:  $surName already exists.  Skipping " .
				"this entry.\n";
	}
	else {

		#
		# Check to see if -p was used. If so, execute txtext and set 
		# remname to the ascii file created
		#
		if ($opt_p){
			print "Executing txtext on $remName\n";
			system ("$txtext -f $remName -p -1 -o $remName.raw"); 
			&filterFile($remName);
			if (!unlink ("$remName.raw")){
				print "pladdsur: Could not remove temporary file " .
						"$remName.raw\n";
			}
			$remName = "$remName.txt";
		}

		#
		#	Open either the file or stdin, based on -i option
		#
		if ($opt_i){
			open (REMFILE, "-") || die "pladdsur: Can't open STDIN";
		}
		else{	
			open (REMFILE, "$remName") || die "pladdsur: Can't open $remName";
		}
	
		open (SURFILE, ">$surName") || die "pladdsur: Can't open $surName";
		
		while (<REMFILE>){
			print SURFILE;
		}
		
		close (REMFILE);
		if ($opt_p) {
			if (!unlink ($remName)){
				print "pladdsur: Could not remove temporary file " .
						"$remName\n";
			}
		}
		close (SURFILE);	
		
		if (-z $surName){
			unlink ($surName);
										### EXIT ###
			die "pladdsur: file is empty.\n";
		}

		#
		#	Add arguments to array (makes it easier to process later
		#
		@surNames[$index]	= $surName;
		@remNames[$index]	= $remName;
		@urls[$index]		= $url;	
		@hl[$index]			= $title;
		$index++;
		$files = 1;
	}
}

if (!$files){
	die "pladdsur:  There are no files to add!  Exiting.\n";
}

#
#	Get path for "admin" directory and prepend it to the pladd and txtext call.
#
$pladd = $adpath . "pladd";

#
#	Run pladd with newlist that was just made.
#
print "Executing pladd\n";

if ($opt_f){
	if ($opt_v){
		print"$pladd -v  -f $tmpList $dbName $surName\n";
		system ("$pladd -v  -f $tmpList $dbName $surName");
	}
	else{
		print"$pladd   -f $tmpList $dbName $surName\n";
		system ("$pladd   -f $tmpList $dbName $surName");
	
	}

	# Remove the temporary list
	if ((system "rm", "$tmpList") >> 8 != 0 ){
		print "Warning:  Could not remove temporary file $tmpList\n";
	}
}
else {
	if ($opt_v){
		print"$pladd -v  $dbName $surName\n";
		`$pladd -v  $dbName $surName\n`;
	}
	else{
		print"$pladd $dbName $surName\n";
		`$pladd  $dbName $surName\n`;
	}
}


#
#	Loop places URL info into surrogate file 
#

for ($i = 0 ; $i < $index; $i++) {

	#
	#	Make the surrogate file, that will replace the original
	#
	open (OUTPUT2, ">@surNames[$i].new") || 	
			die "pladdsur: Can't open @surNames[$i].new";

	select (OUTPUT2);
	$plsMark = "<PLS: URL>";
	$link = "<A HREF=\"@urls[$i]\">@hl[$i]</A>";

	#
	#	Print the special PLS command and the URL to the surrogate file
	#
	print "$plsMark$link\n";

	close (OUTPUT2);
	select (STDOUT);

	#
	#	Replace the original file with the surrogate file
	#
										### EXIT ###
	die "pladdsur: failed to move @surNames[$i].new to @surNames[$i]\n"
		unless rename("@surNames[$i].new", "@surNames[$i]");
}


##############################################################################
#
#   Name:           surUsage
#   Description:    Prints usage of program to the screen 
#   Created:        3/28/95	Bryan Slavin
#   Changed:
#   Returns:        (none)
#	Note:			This routine exits out of the program.
#
##############################################################################

sub surUsage
{
										### EXIT ###
    die <<"EndUsage";
Usage: pladdsur.pl [-i] [-v] [-p] [-f list] Database [URL] [Headline] [Surrogate] [Remote File] 

Parameters:
list		=>	List of remote documents to add
Database	=>	The database in which to add the remote document	
URL		=>	The URL of the remote file
Headline	=>	The headline that will be displayed in the hitlist
Surrogate	=>	Name of the surrogate file that is created
Remote File	=>	Name of the remote file

EndUsage
}

 
##############################################################################
#
#   Name:           getAdPath
#   Description:   	Gets the pathname of the "admin" directory 
#   Created:        ?
#   Changed:
#   Returns:        (none)
#
##############################################################################

sub getAdPath
{
    $adpath = $0;
    if ($adpath ne "pladdsur.pl")
    {
    $adpath =~ s/pladdsur\.pl//ig;
	$adpath = "$adpath"; 
    }
    else
    {
    $adpath = "";
    }
}

##############################################################################
#
#   Name:           filterFile
#   Description:   	Filters out unneeded lines from output
#   Created:        by Chris Eckman
#					Stolen from the original pladdpdf.pl
#   Changed:
#   Returns:        (none)
#
##############################################################################

sub filterFile {

local ($remName) = @_;
local ($remNameNew) = "$remName.txt";
local ($remNameRaw) = "$remName.raw";
local ($counter) = 0;
local ($firstLines) = "";

open (RAWFILE, "$remNameRaw");
open (TXTFILE, ">$remNameNew");

while ( <RAWFILE> ) {

	# Get the filename from the first line of the file
	if ($counter == 0) {
		$pdfFilename = $_;
		chop($pdfFilename);
		$counter++;
		next;
	}

	# Get the number of pages from the second line of the file
	if ($counter == 1) {
		$numPages = $_;
		chop($numPages);
		$numPages =~ s/Number of Pages = //gi;
		$counter++;
		next;
	}

	# Find the first two lines of extracted text, for the headline
	if (($counter == 2)  || ($counter == 3)) {
		next if ( /^\n/ );
		next if ( /^------------- Page \d+ ---------------/ );
		$firstLines .= $_;
		chop($firstLines);
		$counter++;
		next;
	}
 
	#Strip out the page numbers
	s/^------------- Page \d+ ---------------//gi;
 
	$counter++;
 
	print TXTFILE;
}

close (RAWFILE);
close (TXTFILE);

}
