#!/usr/bin/plsperl
#############################################################################
##
##  Personal Library Software 3/8/95
##
##  This work is protected by US Copyright Law and contains proprietary and
##  confidential trade secrets.  PLS (c) Copyright 1995 by PLS,
##  All Rights Reserved
##
##  ### plezdb.pl ###
##
##  Function:  Easily creates and makes a database from a simple argument list
##
##  Created:  3/8/95	Bryan Slavin
##
##  Changed:  4/7/95	Bryan Slavin
##				Cleaned up the code, made minor changes.
##			  4/27/95   Bryan Slavin
##			 	Added a \n to the Show command. Fix for CR 7672
##			  5/3/95	Bryan Slavin
##				Added the capability of extensions as the third argument
##
##############################################################################


##############################################################################
#
#   Name:           mainLine 
#   Description:    -- 
#   Created:        3/8/95	Bryan Slavin 
#   Changed:		12/4/95	Tien Dinh
#				Took out dbgroup option for FastWeb product
#   Returns:        (none)
#
##############################################################################

local ($minArgs) = 1;				# Minimum number of arguments (starting at 0)
local ($defPath);					# Path to the def file
local ($srcDir);					# Directory containing source files
local ($shName);					# Shortname of database created
local ($plcreate, $pladd);			# Path and names of executables

#
# Get paths for admin utils
#
&getAdPath;

#
# Set Flags
#
require "$adpath" . "getopts.pl";
eval '&Getopts (\'g:rv\')';


#
# Set the minimum number of arguments
#
$minArgs = 1;

#
# Get the command-line arguments
#
if ( $#ARGV < $minArgs)
{
    &ezUsage;
}
else
{
	$defPath = shift(@ARGV);
    $srcDir = shift(@ARGV);
	$pattern = shift(@ARGV);
}

#
# Split up pattern variable into an array conataining patterns
#

@patt = split (' ', $pattern);


#
# Set filename for list of source files
#
$tmp = "/tmp/tmp.$$";





#
# Makes list of source files
#
	if (@patt) {
		foreach $patt (@patt){
			if ($opt_r){
				@files = (@files, `find $srcDir -name \"$patt\" -type f -print`);
			}
			else {
				@files = 
					(@files, `find $srcDir -name \"$patt\" -type f -print | grep -v \"$srcDir/.*/\"`);
			}
		}
	}
	else {
		if ($opt_r){
			@files = `find $srcDir -type f -print`;
		}
		else {
			@files = `find $srcDir -type f -print | grep -v \"$srcDir/.*/\"`;
		}
	}


#
# Puts list into a file for pladd
#
die "Could not open $tmp\n" unless open (TMPFILE, ">$tmp");
foreach $_ (@files) {
	print (TMPFILE "$_");
}
close (TMPFILE);

#
# Check to see if $tmp was created
#

if (-z $tmp){
	print "\nNo Source files were found in $srcDir\n";
	print "Check directory and retry\n";
	exit (1);
}


#
# Remove duplicate filenames
#
`sort -u -o $tmp $tmp`;


#
# Adds a .def if there isn't one.
#
if ($defPath !~ /\.def$/){
	$defPath = $defPath . ".def";
}

#
# Creates def file if one doesn't exist 
#
if (! -e $defPath){
	open (DEFFILE, ">$defPath");
	print (DEFFILE "PLWeb/Default	(DST)\n");
	close (DEFFILE);
}


#
# Strip off possible .def in  defPath
#
$defPath =~ s/\.def$//;

#
# Get Shortname from full path if not done recursevly
#
if ($defPath =~ /\//){
	$shName = substr ($defPath, rindex ($defPath, "/") + 1);
}
else {
	$shName = $defPath;
}



$plcreate = $adpath . "plcreate";
$pladd = $adpath . "pladd";

#
# Execute admin utils
#
if ($opt_v){
	print "\n$plcreate -v  $shName -f $defPath\n";
	if (system ("$plcreate -v  $shName -f $defPath") >> 8 != 0) {
		exit 1;															### EXIT ###
	}	
}
else {
	if (system ("$plcreate  $shName -f $defPath") >> 8 != 0) {
		exit 1;															### EXIT ###
	}	
}
if ($opt_v){
	print "$pladd -v -f $tmp  $shName\n";
	if (system ("$pladd -v -f $tmp  $shName") >> 8 != 0) {
		exit 1;															### EXIT ###
	}
}
else {
	if (system ("$pladd -f $tmp  $shName") >> 8 != 0) {
		exit 1;															### EXIT ###
	}
}

`rm $tmp`;


##############################################################################
#
#   Name:           ezUsage
#   Description:    Prints usage of program to the screen 
#   Created:        3/8/95	Bryan Slavin
#   Changed:
#   Returns:        (none)
#
##############################################################################

sub ezUsage
{
    die <<"EndUsage";
Usage: plezdb.pl  [ -rv ]  Def_Path Src_Dir "Patterns"

Parameters:
	-r =>  Look in all directories below "Src Dir" recursively.
	-v =>  Verbose.  Outputs commands used in program.
	Def_Path => Full path and name of the .def file.
	Src_Dir	=> Directory containing the sources for the database.  
		If -r is used, plezdb also adds those files that may 
		be in directories below the Def_Path.
	Patterns => Only add source files that match any one pattern listed
EndUsage
}

 
##############################################################################
#
#   Name:           getAdPath
#   Description:   	Gets the pathname of the "admin" directory 
#   Created:        ?
#   Changed:	    2/29/96	Saad Mufti
#			Try to improve so that it works on NT too.
#   Returns:        (none)
#
##############################################################################

sub getAdPath
{
    $adpath = $0;
    if (!($adpath =~ /^plezdb\.pl$/i))
    {
	$adpath =~ s/plezdb\.pl//i;
	$adpath = "$adpath"; 
    }
    else
    {
	$adpath = "";
    }

    print "The path to directory containing script is <<<$adpath>>>\n";
}
